#!/bin/bash4
#
# Functions to modify the kernel command line in various
# system bootloader configuration files. Written in Bash 4.

UPDATE_GRUB=update-grub

# Modifies the value for the given key in the given configuration
# file. Args:
#
# $1 -- key -- kernel command line option name;
# $2 -- value -- new value;
# $3 -- type -- type of the configuration file ('grub',
#    'boot-conf', 'bootconf', 'extlinux-conf');
# $4 -- file -- configuration file path;
# $5 -- mode -- ('' for set, 'del' for delete, 'print' to
#    output current value).
function edit_cmdline()
{
    KEY="$1"
    VALUE="$2"
    TYPE="$3"
    CONFFILE="$4"
    MODE="$5"

    (
	TEMPFILE=
	if [[ $MODE != "print" ]]
	then
	    TEMPFILE="$(mktemp --tmpdir "$PROG.XXXX")"
	    if [ -z "$TEMPFILE" ]; then
		echo "Unable to create temp file!" >&2
		exit 1
	    fi
	    trap 'rm -f "$TEMPFILE"' EXIT
	fi

        IFS=' '
        while read -r W;
        do
            declare -a str
            read -rd "="  str <<< "$W"

            found=0
            case "$TYPE"  in
                "grub")
                    [ "${str}" = "GRUB_CMDLINE_LINUX_DEFAULT" ] && found=1
                    ;;
                "boot-conf")
                    [ "${str}" = $'\tcmdline' ] && found=1
                    ;;
                "bootconf")
                    [ "${str}" = "bootconf" ] && found=1
                    ;;
                "extlinux-conf")
                    [ "${str}" == $'\tappend root' ] && found=1
                    ;;
            esac

            if [[ "$found"  == "1" ]]
            then
                if [[ "$TYPE" == "grub" ]]
                then
                    start=$(( ${#str} + 2))
                    len=$(( ${#W}   - start - 1 ))
                else
                    start=$((${#str}))
                    len=$(( ${#W}   - start  ))
                fi

                rightpart="${W:$start:$len}"
		arr=($rightpart)
		declare -A new_arr
                used=0

                # Repeat for all  present key-value and keys in
                # command-line string filtered from config:
                for value_pair in "${arr[@]}"
                do
                    IFS="=" read -r k val <<< "$value_pair"
                    if [[ "$k" == "$KEY" ]]
                    then
			if [[ $MODE == "print" ]]
			then
			    # Print the value and exit:
			    echo "$val"
			    exit 0
			fi
                        # A special treatment for lsm, if lsm
                        # is already set, append the given
                        # after a ",":
                        if [[ "$k" == "lsm" ]]
                        then
                            if [[ $MODE != "del" ]]
                            then
                                if [[ $val == *"$VALUE"* ]]
                                then
                                    # already present
                                    new_arr["$k"]="$val"
                                else
                                    new_arr["$k"]="$val,$VALUE"
                                fi
                            else
				if [[ $VALUE != "" ]]
				then
                                    # In del mode with specified
				    # VALUE we need to delete
                                    # the given value from the value
                                    # string.

				    # Convert the value to an array:
                                    IFS=',' read -ra items <<< "${val}"

                                    # Remove $VALUE from the array:
				    declare -A new_items
                                    for item in "${items[@]}"
                                    do
                                        if [[ $item != "$VALUE" ]]
                                        then
                                            new_items["$item"]=1
                                        fi
                                    done

                                    new_string=$(IFS=','; echo "${!new_items[*]}")
                                    # Add remaining to new_arr, if not empty:
				    if [[ $new_string != "" ]]
				    then
					new_arr["$k"]="$new_string"
				    fi
				else
				    # Skip the whole option:
				    :
				fi
                            fi
                        else
                            if [[ $MODE != "del" ]]
                            then
                                if [[ $VALUE == "" ]]
                                then
                                    new_arr["$k"]=
                                else
                                    new_arr["$k"]="$VALUE"
                                fi
                            else
                                # Skip the value in order to delete
                                # it.
				if [[ $VALUE == "" || $VALUE == "$val" ]]
				then
                                    :
				else
				    # Otherwise, keep the current value:
				    new_arr["$k"]="$val"
				fi
                            fi
                        fi

                        # Mark the key as used:
                        used=1
                    else
                        # Pass all other present values to new_arr:
                        if [[ "$used" == 1 && "$k" == "$KEY" ]]
                        then
                            :
                        else
                            if [[ "$val" == "" ]]
                            then
                                new_arr["$k"]=
                            else
                                new_arr["$k"]="$val"
                            fi
                        fi
                    fi
                done # end of value_pair loop

                if [[ "$used" == "0" ]]
                then
                    # We've parsed all the string and have not
                    # found the given key.
		    case "$MODE" in
			print)
			    exit 1
			    ;;
			del)
			    :
			    ;;
			*)
                            if [[ $VALUE == "" ]]
                            then
				new_arr["$KEY"]=
                            else
				new_arr["$KEY"]="$VALUE"
                            fi
			    ;;
                    esac
                fi

                [[ "$TYPE" == "grub" ]] && echo -n "$str='" >> "$TEMPFILE"
                [[ "$TYPE" == "extlinux-conf" ]] && printf "\tappend root=" >>"$TEMPFILE"
                [[ "$TYPE" == "boot-conf" ]] && printf "\tcmdline=" >>"$TEMPFILE"

		sep=
                for k in "${!new_arr[@]}"
                do
                    echo -n "$sep$k" >> "$TEMPFILE"
		    if [[ ${new_arr["$k"]} != "" ]]
		    then
			echo -n "=${new_arr["$k"]}" >> "$TEMPFILE"
		    fi
		    sep=' '
                done
                [[ "$TYPE" == "grub" ]] && echo "'"   >> "$TEMPFILE"
            else
		if [[ $MODE != "print" ]]
		then
                    echo "$W" >> "$TEMPFILE"
		fi
            fi
        done <"$CONFFILE"

	if [[ $MODE != "print" ]]
	then
	    cat "$TEMPFILE" >"$CONFFILE"
	fi
    )
}

# Modifies the given kernel command line option in one
# or more bottloader configuration files. Args:
#
# $1 -- key -- kernel command line option name;
# $2 -- value -- new value;
# $3 -- mode -- ('' for set, 'del' for delete, 'print' to
#    output current value).
function edit_cmdline_all()
{
    if [ -f /etc/sysconfig/grub2 ]
    then
	edit_cmdline "$1" "$2" grub /etc/sysconfig/grub2 "$3" || \
	    return $?
    fi

    if [ -f /boot/boot.conf ]
    then
	edit_cmdline "$1"  "$2" boot-conf /boot/boot.conf "$3" || \
	    return $?
    fi

    if [ -f /boot/bootconf ]
    then
	edit_cmdline "$1"  "$2" bootconf /boot/bootconf "$3" || \
	    return $?
    fi

    if [ -f /boot/extlinux/extlinux.conf ]
    then
	edit_cmdline "$1"  "$2" extlinux-conf /boot/extlinux/extlinux.conf "$3" || \
	    return $?
    fi
}

# Outputs values of the given kernel command line option
# configured in one or more bottloader configuration files.
# The values are sorted in alphabetical order, duplicate
# value are filtered out. Args:
#
# $1 -- key -- kernel command line option name.
function get_cmdline_all()
{
    edit_cmdline_all "$1" '' print | sort -u
}

# Sets the given kernel command line option in one
# or more bottloader configuration files. Args:
#
# $1 -- key -- kernel command line option name;
# $2 -- value -- new value.
function set_cmdline_all()
{
    edit_cmdline_all "$1" "$2"
}

# Deletes the given kernel command line option in one
# or more bottloader configuration files. Args:
#
# $1 -- key -- kernel command line option name;
# $2 -- value -- particular value to delete (optional).
function del_cmdline_all()
{
    edit_cmdline_all "$1" "$2" del
}

# Applies the configuration to the appropriate
# system bootloader.
function apply_bootconf()
{
    local update_grub=
    update_grub="$(which "$UPDATE_GRUB" 2>/dev/null)"

    if [ -n "$update_grub" ]
    then
	"$update_grub"
    fi

    # TODO: Other bootloaders?
}
